﻿using System;
using System.Collections.Generic;
using System.Text;
using Foss.FossDoc.ObjectModel.DataRepresentation.View.DocumentCard;
using Foss.FossDoc.ObjectModel.Data;
using Converters = Foss.FossDoc.ApplicationServer.Converters;
using Foss.FossDoc.ApplicationServer;

// Заменить уникальное имя типа документа (new_library.новый_тип.x20102850)
// на уникальноеимя своего типа документа
namespace Foss.FossDoc.Plugins.DocumentLibrary.new_library.новый_тип.x20102850
{
	public class Form : Foss.FossDoc.ObjectModel.DataRepresentation.View.DocumentCard.DocumentCardInplaceForm
	{
		Foss.FUIS.Action _ActionChangeRights;
		
		public override void Activate()
		{
			base.Activate();

			_GenerateDisplayNameAndDisableField();
		}

		protected override void InitializeActionsButtons()
		{
			_ActionChangeRights = new FUIS.Action();
			_ActionChangeRights.Name = "_ActionChangeRights";
			_ActionChangeRights.Text = "Сменить права";
			_ActionChangeRights.Description = "Сменить права";
			_ActionChangeRights.Perform += _ActionChangeRights_Perform;

			base._ActionManager.AppendAction(_ActionChangeRights);

			base.InitializeActionsButtons();
		}

		protected override void UpdateActionsButtons()
		{
			_ActionChangeRights.Enabled = !IsNew;

			// Если пользователь не находится в группе, например "Право смены прав на объект", то это кнопка прячется
			// Вместо это идентификатора (00000000D04EC1EF9C9E4E1D9E7787DA7F5B8458), вы подставляете идентификатор вашей группы
			if (!ObjectHolder.Session.AccessControlManager.UserManagment.IsUserInGroup(
				ObjectHolder.Session.ActAsUserOID,
				Converters.OID.FromString("00000000D04EC1EF9C9E4E1D9E7787DA7F5B8458")))
			{
				_ActionChangeRights.Visible = false;
			}

			base.UpdateActionsButtons();
		}

		void _ActionChangeRights_Perform(object sender, EventArgs e)
		{
			if (System.Windows.Forms.DialogResult.OK == System.Windows.Forms.MessageBox.Show("Вы уверены, что хотите сменить права на данный документ?", "Смена прав на документ", System.Windows.Forms.MessageBoxButtons.OKCancel, System.Windows.Forms.MessageBoxIcon.Question))
			{
				DS.OID documentOID = ObjectHolder.Id;
				ISession session = ObjectHolder.Session.Session;

				_RecursiveChangeOwner(ObjectHolder.Session.Session, documentOID, ObjectHolder.Session.ActAsUserOID);

				System.Windows.Forms.MessageBox.Show("Права успешно изменены", "Смена прав на документ");
			}
		}

		void _RecursiveChangeOwner(ISession session, DS.OID document, DS.OID newOwner)
		{
			// Получаем владельца объекта
			DS.OID ownerID = session.ObjectDataManager.AccessControlManager.GetOwner(document);

			//получаем все под-объекты и правим их:
			DS.TPropertyTag[][] allContainerTags = session.ObjectDataManager.GetContainerPropertiesTags(new DS.OID[] { document });
			if (allContainerTags != null && allContainerTags.Length > 0)
			{
				DS.TPropertyTag[] containerTags = allContainerTags[0];
				if (containerTags != null && containerTags.Length > 0)
				{
					//теги получены, теперь нам нужно извлечь всех "детей":
					DS.OID[] allChildren = session.ObjectDataManager.GetChildren(new DS.OID[] { document }, containerTags, new DS.TableRestriction(), true, true);
					if (allChildren != null && allChildren.Length > 0)
					{
						//проверяем, кто из этих "детей" имеет родителем текущий документ?
						DS.OID[] childrenParents = session.ObjectDataManager.AccessControlManager.GetParents(allChildren);
						for (int i = 0; i < childrenParents.Length; i++)
						{
							DS.OID child = allChildren[i];
							if (Converters.OID.Equals(childrenParents[i], document)) //только если родитель - наш документ!
							{
								//это кандидат на обработку: ему заменят владельца и сбросят права.
								_RecursiveChangeOwner(session, child, newOwner);
							}
						}
					}
				}
			}

			if (!Converters.OID.Equals(ownerID, newOwner))
			{
				session.ObjectDataManager.AccessControlManager.SetOwner(document, newOwner);
			}

			//сброс прав доступа
			DS.OID[] actors = session.ObjectDataManager.AccessControlManager.GetActors(document);
			if (actors != null && actors.Length > 0)
			{
				session.ObjectDataManager.AccessControlManager.RemoveActors(document, actors);
			}
		}

		private void _GenerateDisplayNameAndDisableField()
		{
			IDocumentCardField documentCardField = Fields[Foss.FossDoc.ApplicationServer.ObjectDataManagment.Schema.PropertyTags.DisplayName];
			if (documentCardField == null)
				return;

			documentCardField.ReadOnly = true;

			if (IsNew)
			{
				documentCardField.Value = _GenerateDisplayName();
			}
		}

		private string _GenerateDisplayName()
		{
			string employeeName;
			using (IObjectHolder employee = ObjectHolder.Session.OpenObject(ObjectHolder.Session.ActAsUserOID))
			{
				employeeName = employee.DisplayName;
			}

			return string.Format(
					"{0} ({1}) {2}",
					ObjectHolder.Category.DisplayName,
					employeeName,
					ObjectHolder.Session.UTCTime.ToLocalTime());
		}

		protected override void SaveRoutine()
		{
			base.SaveRoutine();
		}
	}
}
